/**************************************************************************
 FILE:
	osinm.h

 DESCRIPTION:
	Network mangement-specific.

	@(#)osinm.h 1.25 92/06/12 Copyr 1986 Sun Micro
 **************************************************************************/

/*
 * Architecture:
 * Messages:
 * Parameters:
 */

/*
 * Global required by the kernel network management agent.
 */

struct osinm_stats {
	/* Pointers to arrays of statistics, and the storage size (in bytes) */
	u_long		*stats[OSI_MAX_LAYERS];
	u_short		stat_size[OSI_MAX_LAYERS];

	/* Pointers to protocol switch entries */
	struct protosw	*proto[OSI_MAX_LAYERS];
	struct protosw	*osinm_proto;

	/* The network management process -- NULL if not running */
	struct socket	*so;
};

/*
 * Messages between network management and protocol layers, and
 * between user processes and the kernel network management functions.
 * Most messages are passed with associated parameters as data.
 * "Event Notification" messages are passed with the specific event
 * identification and any associated information as data.
 * NB:	Currently only RR, RC, SR are supported.  SC is implicit
 *	in the return from an ioctl() from the network management process
 *	to the kernel.
 */

typedef enum {
	OSINM_RR,	/* Read Request (RR) */
	OSINM_RC,	/* Read Confirm (RC) */
	OSINM_SR,	/* Set Request (SR) */
	OSINM_SC,	/* Set Confirm (SC) */
	OSINM_CSR,	/* Compare and Set Request (CSR) */
	OSINM_CSC,	/* Compare and Set Confirm (CSC) */
	OSINM_EN,	/* Event Notification (EN) */
	OSINM_AR,	/* Action Request (AR) */
	OSINM_AC,	/* Action Confirm (AC) */
	OSINM_ER	/* Error Response (ER) */
} OSINM_MESSAGE_TYPE;

typedef enum {
	OSINM_CL_CONFIG,	/* Configuration class */
	OSINM_CL_PERFORM,	/* Performance class */
	OSINM_CL_FAULT,		/* Fault class */
	OSINM_CL_EVENT		/* Event class */
} OSINM_MESSAGE_CLASS;
	
typedef struct {
	CTL_EVENT		ctl_event;
	CTL_LAYER		layer;
	OSINM_MESSAGE_TYPE	message_type;
	OSINM_MESSAGE_CLASS	message_class;
} CTL_MSG_MNGMT;


/*
 * Parameter values in each of the message classes.
 */


/* 
 * Network Layer parameters (performance class) 
 */

typedef enum {
	NM_IND_SDU_SENT			= 0,
					/* CLNP NSDU's sent */
	NM_IND_SDU_RECV			= 1,
					/* CLNP NSDU's received */
	NM_IND_ACTIVE_SENT		= 2,
					/* Active packets sent */
	NM_IND_ACTIVE_RECV		= 3,
					/* Active packets received */
	NM_IND_INACTIVE_SENT		= 4,
					/* Inactive packets sent */
	NM_IND_INACTIVE_RECV		= 5,
					/* Inactive packets received */
	NM_IND_ESH_SENT			= 6,
					/* ES-IS ESH packets sent */
	NM_IND_ESH_RECV			= 7,
					/* ES-IS ESH packets received */
	NM_IND_ISH_SENT			= 8,
					/* ES-IS ISH packets sent */
	NM_IND_ISH_RECV			= 9,
					/* ES-IS ISH packets received */
	NM_IND_RD_SENT			= 10,
					/* ES-IS RD packets sent */
	NM_IND_RD_RECV			= 11,
					/* ES-IS RD packets received */
	NM_IND_ERRORPDUS_RECV		= 12,
					/* Error PDU's received */
	NM_IND_DSTADDR_DROPPED		= 13,
					/* Dst Address doesn't exist */
	NM_IND_CHECKSUM_DROPPED		= 14,
					/* Bad checksum -- dropped */
	NM_IND_ESIS_CHECKSUM		= 15,
					/* ES-IS with bad checksum packets 
					 * received */
	NM_IND_OPNOSUPP_DROPPED		= 16,
					/* Options not supported */
	NM_IND_LIFEEXCEEDED_DROPPED 	= 17,
					/* Lifetime exceeded in transit */
	NM_IND_REASS_DROPPED		= 18,
					/* Lifetime exceeded during reassembly */
	NM_IND_CONGESTION_DROPPED	= 19,
					/* Congestion -- dropped */
	NM_IND_NSEL_DROPPED		= 20,
					/* No one registered on the NSEL  */
	NM_IND_BAD			= 21,
					/* Bad packet */
	NM_IND_BAD_CLNP			= 22,
					/* Bad CLNP packet */
	NM_IND_ESIS_BAD			= 23,
					/* Bad ES-IS packets received */
	NM_IND_BAD_LENGTH		= 24,
					/* Too short packet */
	NM_IND_BAD_HLEN			= 25,
					/* Too long CLNP or ES-IS header */
} OSINM_IND_PERF_PARAMS;

#define NM_IND_MAX_PERF_PARAMS		26

/*
 * Transport Layer parameters (performance class).
 */

typedef enum {
	NM_TP_CR_INREJECTED		= 0,
					/* CR's received that were rejected */
	NM_TP_CR_OUTREJECTED		= 1,
					/* CR's sent that were rejected */
	NM_TP_CR_SENT			= 2,
					/* CR's sent */
	NM_TP_RETRANS_CR		= 3,
					/* The number of CR's that required
					 * retransmission */
	NM_TP_INCONNECTIONS		= 4,
					/* Number of connections successfully
					 * completed with RECEPTION of a CC */
	NM_TP_OUTCONNECTIONS		= 5,
					/* Number of connections successfuly
					 * completed with OUTGOING CC */
	NM_TP_TPDU_INVALID		= 6,
					/* Number of invalid TPDU's received */
	NM_TP_RETRANS_DATA		= 7,
					/* Number of TPDU's that required
					 * retransmission during the TS_OPEN
					 * state */
	NM_TP_OCTETS_DTSENT		= 8,
					/* Total number of octets of user
					 * data sent */
	NM_TP_OCTETS_DTRECV		= 9,
					/* Total number of octets of user
					 * data received */
	NM_TP_OCTETS_EDSENT		= 10,
					/* Total number of octets of expedited
					 * data sent */
	NM_TP_OCTETS_EDRECV		= 11,
					/* Total number of octets of expedited
					 * data received */
	NM_TP_RETRANS_PDU		= 12,
					/* Total PDU retransmission count */
	NM_TP_TIMER_INACTIVITY		= 13,
					/* Total number of connections
					 * terminated due to expiration of
					 * the inactivity timer */
	NM_TP_DR_SENT			= 14,
					/* Number of requests sent to disconnect
					   from network */
	NM_TP_RETRANS_AK		= 15,
					/* Total number of AK's retransmitted*/
					/* NB:  Not currently used */
	NM_TP_REFUSED_TYPE1		= 16,
					/* Connection refused -- type 1 --
					 * insufficient free space */
					/* NB:  Not currently used */
	NM_TP_REFUSED_TYPE2		= 17,
					/* Connection refused -- type 2 --
					 * number of connections would exceed
					 * maximum */
					/* NB:  Not currently used */
	NM_TP_NOPEN			= 18,
					/* Number of connections currently
					 * open */
	NM_TP_PROT_ERRORS		= 19,
					/* Number of protocol errors */
	NM_TP_SENT_PDU			= 20,
					/* Total number of PDU's transmitted */
	NM_TP_RECV_PDU			= 21,
					/* Total number of received PDU's */
	NM_TP_CHECKSUM_DROPPED		= 22,
					/* Dropped PDU's from bad checksum */
	NM_TP_ER_SENT			= 23,
					/* Number of TP error packets sent */
} OSINM_TP_PERF_PARAMS;

#define NM_TP_MAX_PERF_PARAMS		24

/*
 * Transport layer parameters (configuration class).
 */

typedef enum {
	NM_TP_TIME_AK			= 0,
					/*Maximum AK time from receipt of DT */
	NM_TP_TIME_RETRANS_MAX		= 1,
					/* Maximum delay for retransmission */
	NM_TP_TIME_RETRANS_START	= 2,
					/* Starting delay for retransmission */
	NM_TP_TIME_INACTIVITY		= 3,
					/* Inactivity timer */
	NM_TP_TIME_WINDOW		= 4,
					/* Window timer */
	NM_TP_TIME_REFERENCE		= 5,
					/* Reference timer */
	NM_TP_HIWAT_SEND		= 6,
					/* Hi-water mark on send queue */
	NM_TP_HIWAT_RECV		= 7,
					/* Hi-water mark on receive queue */
	NM_TP_LOWAT_SEND		= 8,
					/* Lo-water mark on send queue */
	NM_TP_LOWAT_RECV		= 9,
					/* Lo-water mark on receive queue */
	NM_TP_IFCUNIT_DEFAULT		= 10,
					/* Default interface data unit size
					 * to the transport impl. */
	NM_TP_NSAP_ID_DEFAULT		= 11,
					/* Add this NSAP ID to valid NSAP's */
	NM_TP_NSAP_ID_ADD		= 12,
					/* Add this NSAP ID to valid NSAP's
					 * on which Transport accepts
					 * PDU's */
	NM_TP_NSAP_ID_DELETE		= 13,
					/* Delete this NSAP ID */
	NM_TP_DEL_ACK_FRACTION		= 14,
					/* Fraction of window at which to 
					 * acknowledge packets.  0 turns off
					 * delayed AK, 10 waits for window
					 * to be filled.  Values of n between
					 * 1 and 9 (inclusive) turn on delayed
					 * AK for the fraction n/10 */
	NM_TP_MAX_PDUSIZE		= 15,
					/* Maximum PDU size for this node */
	NM_TP_MAX_RETRANSMISSIONS	= 16,
					/* Maximum number of retransmissions
					 * before release procedure */
	NM_TP_CHECKSUM			= 17,
					/* 1 for checksums on, 0 for off */
	NM_TP_EXTENDED_FORMATS		= 18,
					/* 1 for extended formats, 0 for 
					 * normal formats */
	NM_TP_STARTING_REFERENCE	= 19,
					/* value at which to start local
					 * reference id -- chosen by the
					 * clock otherwise */
	NM_TP_TIME_TS1			= 20,
					/* The default TS1 connect timer 
					 * for TP0/TP2. */
	NM_TP_TIME_TS2			= 21,
					/* The default TS2 disconnect timer 
					 * for TP2. */
	NM_TP_PROPOSE_TP2		= 22,
					/* Propose TP2/TP0 or just TP0
					 * over CONS. */
/* ICNCFIX by -TH- add field to turn off the piggybacking of ACKs.  This messes
   up one of the conformance tests.  Leave this as an undocumented feature so
   as to maintain backward compatibility with docs. */
	NM_TP_NO_PIGGYBACK		=23,
/* End of fix */
					 
} OSINM_TP_CONF_PARAMS;

/* ICNCFIX by -TH- had to increment this since I added a new parameter
#define NM_TP_MAX_CONF_PARAMS		23*/
#define NM_TP_MAX_CONF_PARAMS           24

/*
 * Session Layer parameters (performance class).
 */


typedef enum {
	NM_SES_CONN_SENT		= 0,
					/* Number of session connections
					 * initiated */
	NM_SES_CONNECT_ACCEPTED		= 1,
					/* Number of session connections
					 * accepted locally */
	NM_SES_CONNECT_REJECTED		= 2,
					/* Number of session connections
					 * rejected locally */
	NM_SES_ABORT_SENT		= 3,
					/* Number of aborts sent */
	NM_SES_ABORT_RECEIVED		= 4,
					/* Number of aborts received */
	NM_SES_RELEASE_SENT		= 5,
					/* Number of orderly releases sent */
	NM_SES_RELEASE_RECEIVED		= 6,
					/*Number of orderly releases received*/

} OSINM_SES_PERF_PARAMS;

#define NM_SES_MAX_PERF_PARAMS		7

/*
 * Session Layer parameters (configuration class).
 */

typedef enum {
	NM_SES_TIMER			= 0,
					/* Session's timer */
} OSINM_SES_CONF_PARAMS;

#define NM_SES_MAX_CONF_PARAMS		1

/*
 * CASE Layer parameters.
 */

typedef enum {
	NM_CASE_OPENED			= 0,
					/* Number of "open"ed associations */
	NM_CASE_CLOSED			= 1,
					/* Number of "close"ed associations */
	NM_CASE_PROT_VIOLATIONS		= 2
					/* Number of protocol violations */
} OSINM_CASE_PERF_PARAMS;

#define NM_CASE_MAX_PERF_PARAMS		3

/*
 * Macros by which layer implementations record statistics with the network
 * management agent on the local node.  Parametes can be performance class
 * parameters only.
 */

#define osinm_ind_counter( parameter ) \
		(((osinm_stats.stats[(int)OSI_IND])[(int)parameter])++)
#define osinm_tp_counter( parameter ) \
		(((osinm_stats.stats[(int)OSI_TP])[(int)parameter])++)
#define osinm_tp_decrementer( parameter ) \
		(((osinm_stats.stats[(int)OSI_TP])[(int)parameter])--)
#define osinm_ses_counter( parameter ) \
		(((osinm_stats.stats[(int)OSI_SES])[(int)parameter])++)

#define osinm_ind_stats( parameter, value ) \
		(((osinm_stats.stats[(int)OSI_IND])[(int)parameter]) += (value))
#define osinm_tp_stats( parameter, value ) \
		(((osinm_stats.stats[(int)OSI_TP])[(int)parameter]) += (value))
#define osinm_ses_stats( parameter, value ) \
		(((osinm_stats.stats[(int)OSI_SES])[(int)parameter]) += (value))

/*
 * Asynchronous event notification by protocol layers.
 * NB:	Threshold values are not implemented as asynchronous events --
 *	rather, the network management process polls at a tunable interval
 *	for statistics and determines when a threshold value has been
 *	passed.
 */


#define osinm_cons_event(event, pinfo, size) \
		(void) osinm_event(OSI_CONS, event, pinfo, size)

#define osinm_ind_event(event, pinfo, size) \
		(void) osinm_event(OSI_IND, event, pinfo, size)

#define osinm_tp_event(event, pinfo, size) \
		(void) osinm_event(OSI_TP, event, pinfo, size)

#define osinm_ses_event(event, pinfo, size) \
		(void) osinm_event(OSI_SES, event, pinfo, size)


/*
 * Available "Event Notification" messages.  These are passed between
 * layers and between kernel and user as follows:  OSINM_TYPE followed
 * by identifying information (e.g. OSINM_EN_NODSTADDR might be followed
 * by the address of the remote requesting entity).
 */


typedef enum {
	OSINM_EN_ABORT		= 1,
				/* Abnormal abort of established connection */
	OSINM_EN_NODSTADDR	= 2,
				/* Destination address doesn't exist */	
	OSINM_EN_PROTOVIOLA	= 3,
				/* Protocol violation */
	OSINM_EN_OPNOTSUPP	= 4,
				/* Unsupported options (or PDU) */
	OSINM_EN_BADCONNECTPDU	= 5,
				/* CASE: Bad A-ASSOCIATE PDU */
	OSINM_EN_IGNORANTUSER	= 6,
				/* CASE: "Indication that the originator of
				 * message is not aware of the services
				 * provided at the SAP's to which it is bound
				 */
} OSINM_EN_TYPE;

/* 
 * Structures for asynchronous notification. 
 */ 
 
#ifdef _TP_EVENT
struct osinm_tp_nodstaddr {
	OSINM_EN_TYPE	en_type;
	XSEL		tsel;
};
#endif _TP_EVENT

#ifdef _SES_IMPL
struct osinm_ses_nodstaddr {
	OSINM_EN_TYPE		en_type;
	XSEL		ssap;
};
#endif _SES_IMPL

#ifdef KERNEL
extern struct osinm_stats   osinm_stats;
#endif KERNEL

